{*******************************************************}
{                                                       }
{       I/O functions - Implementation Part             }
{       -----------------------------------             }
{       ADV Compiled                                    }
{       Copyright (C), Lucian Radulescu                 }
{       E-mail:    ljee@interlog.com                    }
{       Home Page: http://www.interlog.com/~ljee        }
{                                                       }
{*******************************************************}

unit FileTool;

{$I FILETOOL.INT}

implementation

uses
  {$IFDEF WIN32}
  Registry,
  {$ELSE}
  RegInc, Regi1632,
  {$ENDIF}
  EQ_Abort, WinTools;

const
  fsPathName = 79;

const
  {magic number used when compressing with LZH}
  LZHIdentifier:  Longint = $21485A4C; {LZH!}

var
  DosError: Integer;

function ResetIOResult: Integer;
begin
  Result := IOResult;
end;

function OpenTextFile(var F: TextFile; const FileName: String; oft: TOpenFileType): Boolean;
begin
  ResetIOResult;
  AssignFile(F, FileName);
  case oft of
    topRead   : Reset(F);
    topWrite  : Rewrite(F);
    topAppend : Append(F);
  end;
  Result := IOResult = 0;
end;

function GetFileSize(const FileName: String): LongInt;
var
  Handle: Integer;                        { working file           }
  OldErrorMode: Word;
begin                                     { begin body             }
  ResetIOResult;                          { Reset IOResult         }
  OldErrorMode := SetErrorMode(SEM_FAILCRITICALERRORS);
  try
    Handle:=FileOpen(FileName,fmOpenRead);{ try to open file       }
    if Handle<0 then Result:=0            { return 0 on error      }
    else Result:=FileSeek(Handle,0,2);    { get file size          }
    FileClose(Handle);                    { close file             }
  finally
    SetErrorMode(OldErrorMode);
  end;
end;                                      { end body               }

{ Avoid using this function for files bigger than 64KB }
function ReadFileToBuffer(FileName: String): PChar;
var
  FSize: Cardinal;                        { size of file           }
  F: File;                                { working file           }
begin                                     { begin body             }
  Result := nil;                          { prepare for worse      }
  FSize := GetFileSize(FileName);         { get file size          }
  if (FSize=0) then Exit;                 { exit if not valid      }
  Result := GetClearStr(FSize+1);         { allocate&clean memory  }
  ResetIOResult;                          { Reset IOResult         }
  AssignFile(F, FileName);                { assign name to file    }
  {$I-}                                   { set I/O checking off   }
  Reset(F, 1);                            { open file, read mode   }
  {$I+}                                   { set I/O checking on    }
  if IOResult = 0 then                    { succes                 }
  try                                     { bb                     }
    BlockRead(F,Result^,FSize);           { read                   }
  finally
    CloseFile(F);                         { close file             }
  end                                     { eb                     }
  else MyStrDispose(Result);
end;                                      { end body               }

function ReadFileInBuffer(FileName: String; Buffer: PChar; Size: Word): PChar;
var
  FSize: Cardinal;                        { size of file           }
  F: File;                                { working file           }
begin                                     { begin body             }
  Result := nil;                          { prepare for worse      }
  if not FileExists(FileName) then Exit;  { no such file           }
  FSize := GetFileSize(FileName);         { get file size          }
  if (FSize=0) then Exit;                 { exit if not valid      }
  if FSize > Size then FSize := Size;     { resize if necessary    }
  ResetIOResult;                          { Reset IOResult         }
  AssignFile(F, FileName);                { assign name to file    }
  {$I-}                                   { set I/O checking off   }
  Reset(F, 1);                            { open file, read mode   }
  {$I+}                                   { set I/O checking on    }
  if IOResult = 0 then                    { succes                 }
  try                                     { bb                     }
    BlockRead(F,Buffer^,FSize);           { read                   }
    Result := Buffer;                     { just to behave nice    }
  finally
    CloseFile(F);                         { close file             }
  end;                                    { eb                     }
end;                                      { end body               }

function WriteFileFromBuffSized(FileName: String; Buffer: PChar; Size: Cardinal): Boolean;
var
  F: File;                                { working file           }
  BytesDone: Integer;                     { number of bytes copied }
begin                                     { begin body             }
  Result := Buffer <> nil;                { nil buffer             }
  if not Result then Exit;                { out if empty           }
  ResetIOResult;                          { Reset IOResult         }
  AssignFile(F, FileName);                { assign name to file    }
  {$I-}
  Rewrite(F, 1);                          { rewrite file           }
  {$I+}
  if IOResult = 0 then                    { success                }
  try
    BlockWrite(F,Buffer^,Size,BytesDone); { write file             }
    Result := BytesDone = Size;           { transfer correct       }
  finally
    CloseFile(F);                         { close file             }
  end;
end;                                      { end body               }

function WriteFileFromBuff(FileName: String; Buffer: PChar; LastByte: Boolean): Boolean;
var
  F: File;                                { working file           }
  BytesToCopy: Integer;                   { bytes to copy          }
  BytesDone: Integer;                     { bytes copied           }
begin                                     { begin body             }
  Result := not NilPtr(Buffer);           { buffer empty ?         }
  if not Result then Exit;                { out if empty           }
  ResetIOResult;                          { Reset IOResult         }
  AssignFile(F, FileName);                { assign name to file    }
  {$I-}Rewrite(F, 1);{$I+}                { rewrite file           }
  if IOResult = 0 then                    { success                }
  try
    BytesToCopy := StrLen(Buffer);        { what's the size?       }
    if LastByte then                      { this will copy the ending }
      Inc(BytesToCopy);                   { null character.           }
    BlockWrite(F,Buffer^,BytesToCopy,BytesDone); { write file      }
    Result := BytesDone = BytesToCopy;    { transferred correct    }
  finally
    CloseFile(F);                         { close file             }
  end;
end;                                      { end body               }

function ReturnDirectory(const APath, ADir: String): String;
var
  NewDir: String;
begin
  if pos(ADir, APath) > 0 then
  begin
    Result := APath;
    Exit;
  end;
  NewDir := APath;
  if pos('\', NewDir) < Length(NewDir) then NewDir := NewDir + '\' + ADir
  else NewDir := NewDir + ADir;
  Result := NewDir;
end;

{ In Delphi 32 bits, the function DeleteFile is declared in both Windows and SysUtils, but
  with different type of parameter.
  The way that Delphi creates projects and forms, in uses clause, it puts SysUtils before
  Windows (32 bits) or before WinProcs (16 bits), this way makes us think it's a bugg with
  DeleteFile function. All you have to do is use DeleteFile with unitname in front like
  SysUtils.DeleteFile or Windows.DeleteFile, or if not, make sure SysUtils follows Windows. }
function EraseFile(FileName: String): Boolean;
begin
  FileSetAttr(FileName, faArchive);
  Result := SysUtils.DeleteFile(FileName);
end;

{$IFNDEF WIN32}
procedure AnsiDosFunc; assembler;
var
  TempName: array[0..fsPathName] of Char;
asm
	PUSH	DS
	PUSH	CX
	PUSH	AX
	MOV	SI,DI
	PUSH	ES
	POP	DS
	LEA	DI,TempName
	PUSH	SS
	POP	ES
	MOV	CX,fsPathName
	CLD
@@1:	LODSB
	OR	AL,AL
	JE	@@2
	STOSB
	LOOP	@@1
@@2:	XOR	AL,AL
	STOSB
	LEA	DI,TempName
	PUSH	SS
	PUSH	DI
	PUSH	SS
	PUSH	DI
	CALL	AnsiToOem
	POP	AX
	POP	CX
	LEA	DX,TempName
	PUSH	SS
	POP	DS
	INT	21H
	POP	DS
end;

procedure CreateDir(Dir: PChar); assembler;
asm
	LES	DI,Dir
	MOV	AH,39H
	CALL	AnsiDosFunc
	JC	@@1
	XOR	AX,AX
@@1:	MOV	DosError,AX
end;

procedure RemoveDir(Dir: PChar); assembler;
asm
	LES	DI,Dir
	MOV	AH,3AH
	CALL	AnsiDosFunc
	JC	@@1
	XOR	AX,AX
@@1:	MOV	DosError,AX
end;
{$ENDIF}

{ DeleteAllFiles will delete ALL files: readonly, hidden, system (!).
  Returns True in case of success.
  Returns False if wrong Directory name.

  *After succesfully calling this function, Directory is EMPTY:
    (no more files with this attributes:
      faReadOnly + faHidden + faSysFile + faArchive).

  Directory should look like this:
  1. c:\a\b\c\alfa.*  - directory + filename + wildcard extension
  2. c:\a\b\c\        - directory only (backslash at the end)
  3. c:\a\b\c         - directory only (without backslash ath the end)
}
function DeleteAllFiles(Directory: String): Boolean;
const
  Attr: Word = faReadOnly + faHidden + faSysFile + faArchive;
var
  SR: TSearchRec;
  K, Res: Integer;
  InitDir: String;
  HasWild: Boolean;
begin
  GetDir(0, InitDir);
  Result := False;

  HasWild := pos('*', Directory) > 0;
  if not HasWild then {$I-}ChDir(Directory){$I+}
  else
  begin
    K := Length(Directory);
    while Directory[K] <> '\' do Dec(K);
    {$I-}ChDir(Copy(Directory, 1, K));{$I+}
  end;
  if IOResult <> 0 then Exit; { wrong directory }

  Result := True;
  if not HasWild then Res := FindFirst('*.*', Attr, SR)
  else
    Res := FindFirst(Directory, Attr, SR);
  while Res = 0 do
  begin
    Result := EraseFile(SR.Name) and Result;
    Res := FindNext(SR);
  end;
  FindClose(SR);
  ChDir(InitDir);
end;

{ DeleteDirectory will delete Directory and all subdirectories and files.
  (See DeleteAllFiles comments regarding deletion of files)
  Returns True in case of success.
  Returns False if wrong Directory name, no file(s) deleted.
  Returns False if length of Directory > fsPathName, but subdirectories ARE deleted.

  *After succesfully calling this function, Directory is DELETED.

  Directory should look like this:
  1. c:\a\b\c\
  2. c:\a\b\c
}
function DeleteDirectory(Directory: String): Boolean;
var
  SR: TSearchRec;
  InitDir, NewDir: String;
  Res: Integer;
  DelDir: array [0..fsPathName] of char;
begin
  Result := False;
  if Length(Directory) <= 3 then Exit;
  if Directory[Length(Directory)] <> '\' then
    Directory := Directory + '\';
  GetDir(0, InitDir);

  Result := DeleteAllFiles(Directory);
  {$I-}ChDir(Directory);{$I+}
  if IOResult <> 0 then
  begin
    Result := False;
    Exit;
  end;

  NewDir := Directory + '*.*';
  Res := FindFirst(NewDir, faDirectory, SR);
  while Res = 0 do
  begin
    if (SR.Attr and faDirectory <> 0) and (SR.Name[1] <> '.') and (SR.Name <> Directory) then
      Result := DeleteDirectory(Directory+SR.Name+'\') and Result;
    Res := FindNext(SR);
  end;
  FindClose(SR);

  if Length(Directory) > fsPathName then
  begin
    Result := False;
    Exit;
  end;

  FillChar(DelDir, SizeOf(DelDir), #0);
  Move(Directory[1], DelDir[0], Length(Directory) - 1);
  {DelDir should be empty by now}
  RemoveDir(DelDir);
  Result := Result and (DOSError=0);
  ChDir(InitDir);
end;

function SearchFile(StartPath, FileName: PChar; SubDirs: Boolean): Boolean;
var
  NewDir: String;
  SR: TSearchRec;
  Res: Integer;
  Dir: array [0..12] of Char;
  Len: Word;
begin
  if (StrEnd(StartPath)-1)^ = '\' then (StrEnd(StartPath)-1)^ := #0;
  Result := FileExists(Format('%s\%s', [StartPath, FileName]));
  if not SubDirs or Result then Exit;
  NewDir := Format('%s\*.*', [StartPath]);
  Res := FindFirst(NewDir, faDirectory, SR);
  while Res = 0 do
  begin
    if (SR.Attr and faDirectory <> 0) and (SR.Name[1] <> '.') then
      begin
	Len := StrLen(StartPath);
	Result := SearchFile(StrCat(StrCat(StartPath, '\'), StrPCopy(Dir, SR.Name)), FileName, True);
	if not Result then StartPath[Len] := #0;
      end;
    if Result then Break;
    Res := FindNext(SR);
  end;
  FindClose(SR);
end;

{warning: not checked for OpenFile failure }
procedure TouchFile(FileName: String; DateTime: Longint);
var
  FileHandle: Integer;
  OpenBuf: TOFStruct;
  FileNameCh: array [0..255] of char;
begin
  StrPCopy(FileNameCh, FileName);
  FileHandle := OpenFile(FileNameCh, OpenBuf, OF_SHARE_EXCLUSIVE or OF_READWRITE);
  FileSetDate(FileHandle, DateTime);
  _lClose(FileHandle);
end;

{warning: not checked for OpenFile failure }
function GetFileDateTime(FileName: String): TDateTime;
var
  FileHandle: Integer;
  OpenBuf: TOFStruct;
  FileNameCh: array [0..255] of char;
  LongDate: LongInt;
begin
  StrPCopy(FileNameCh, FileName);
  FileHandle := OpenFile(FileNameCh, OpenBuf, OF_READ);
  LongDate := FileGetDate(FileHandle);
  _lClose(FileHandle);
  Result := FileDateToDateTime(LongDate);
end;

function MSUsed(PathName: PChar): Boolean;
var
  Name: array [0..255] of char;
begin
  if StrRScan(PathName, '_') = (StrEnd(PathName)-1) then
  begin
    FillChar(Name, SizeOf(Name), #0);
    GetExpandedName(PathName, Name);
  end
  else StrCopy(Name, PathName);
  Result := StrIComp(PathName, Name) <> 0;
end;

function xOpenFile(PathName: PChar; ReadWrite: Integer; ExpandCompressed: Boolean): Integer;
var
  OpenBuf: TOFStruct;
begin
  if ExpandCompressed and MSUsed(PathName) then
    Result := LZOpenFile(PathName, OpenBuf, ReadWrite)
  else
    Result := _lopen(PathName, ReadWrite);
end;

function xSeek(FileHandle: Integer; SeekTo: Longint;
  Mode: Integer; ExpandCompressed: Boolean): Longint;
begin
  if ExpandCompressed then
    Result := LZSeek(FileHandle, SeekTo, Mode)
  else
    Result := _llseek(FileHandle, SeekTo, Mode);
end;

function xClose(FileHandle: Integer; ExpandCompressed: Boolean): Integer;
begin
  Result := 0;
  if ExpandCompressed then
    LZClose(FileHandle)
  else
    Result := _lclose(FileHandle);
end;

function xRead(FileHandle: Integer; Buf: PChar; Count: Integer; ExpandCompressed: Boolean): Integer;
begin
  if ExpandCompressed then
    Result := LZRead(FileHandle, Buf, Count)
  else
    Result := _lread(FileHandle, Buf, Count);
end;

function xGetFileSize(PathName: PChar; ExpandCompressed: Boolean): Longint;
var
  SourceHandle: Integer;
  Identifier: LongInt;
begin
  if not ExpandCompressed then
    Result := GetFileSize(StrPas(PathName))
  else
    if MSUsed(PathName) then
    begin
      SourceHandle := xOpenFile(PathName, of_Share_Deny_None or of_Read, ExpandCompressed);
      Result := xSeek(SourceHandle, 0, 2, ExpandCompressed);
      xClose(SourceHandle, ExpandCompressed);
    end
    else
    begin
      SourceHandle := xOpenFile(PathName, of_Share_Deny_None or of_Read, False);
      FileRead(SourceHandle, Identifier, Sizeof(Longint));
      if Identifier = LZHIdentifier then
        FileRead(SourceHandle, Result, Sizeof(Longint))
      else
        Result:=xSeek(SourceHandle,0,2,False);
      xClose(SourceHandle, False);
    end;
end;

function MultiFileExistsOR(PathName: String; Files: array of String): Boolean;
var
  I: Integer;
begin
  Result := False;
  for I := Low(Files) to High(Files) do
  begin
    Result := Result or FileExists(AddBackSlash(PathName)+Files[I]);
    if Result then Break;
  end;
end;

function MultiFileExistsAND(PathName: String; Files: array of String): Boolean;
var
  I: Integer;
begin
  Result := True;
  for I := Low(Files) to High(Files) do
  begin
    Result := Result and FileExists(AddBackSlash(PathName)+Files[I]);
    if not Result then Break;
  end;
end;

function fOpenCheck(var F: File; AFileName: String; ForRead: Boolean): Boolean;
begin
  Result := not ForRead or FileExists(AFileName); {if read request and not fileexist}
  if not Result then Exit;                     {return false    }

  AssignFile(F, AFileName);                    { Assign name    }
  {$I-}
  if ForRead then Reset(F, 1)                  { Open for read  }
  else Rewrite(F, 1);                          { Open for write }
  {$I+}
  Result := IOResult = 0;                      { Check IOResult }
end;

function GetVersionData(AFileName, QueryValue: String; Buff: PChar): PChar;
{$IFNDEF WIN32}
type
  DWORD = LongInt;
{$ENDIF}
var
  Buffer: Pointer;
  FileName: array [0..255] of char; { Filename of version stamped file }
  QueryVal: array [0..255] of char; { Query value as PChar }
  VerSize: LongInt;                 { Length of buffer for info }
  PData: PChar;                     { Buffer to place the data structure }
  VersionHandle: DWORD;             { Information for use by GetFileVersionInfo }
  {$IFDEF WIN32} Len: DWORD;        { Length, in bytes, of the version-information value. }
  {$ELSE}        Len: Word;         { Length, in bytes, of the version-information value. }
  {$ENDIF}
begin
  StrPCopy(FileName, ExpandFileName(AFileName));
  StrPCopy(QueryVal, QueryValue);
  VerSize := GetFileVersionInfoSize(FileName, VersionHandle);
  StrCopy(Buff, #0);
  if VerSize > 0 then
  try
    PData := GetClearStr(VerSize+1);
    if GetFileVersionInfo(FileName, VersionHandle, VerSize, PData) then
      if VerQueryValue(PData, QueryVal, Buffer, Len) then
        StrCopy(Buff, Buffer);
  finally
    MyStrDispose(PData);
  end;
  Result := Buff;
end;

function GetResourceInfoStr(ResName: String): String;
var
  Buffer: PChar;
begin
  Buffer := StrAlloc(1000);
  FillChar(Buffer^, 1000, #0);
  try
    Result := StrPas(GetVersionData(Application_ExeName, ResName, Buffer));
  finally
    StrDispose(Buffer);
  end;
end;

function GetModuleResourceInfoStr(const ModuleName, ResName: String): String;
var
  Buffer: PChar;
begin
  Buffer := StrAlloc(1000);
  FillChar(Buffer^, 1000, #0);
  try
    Result := StrPas(Buffer);
    Result := StrPas(GetVersionData(ModuleName, ResName, Buffer));
  finally
    StrDispose(Buffer);
  end;
end;

function DriveIsReady(Drive: Char; FileMode: Integer; var Abort: Boolean;
  var erCode: Integer): Boolean;
const
  Csts: array [Boolean] of String = ('not ready.', 'write protected.');
var
  OpenBuff: TOfStruct;
  OldErrorMode: Word;
  CheckWrite: Boolean;
  FName: array [0..15] of char;
  Msg: array [0..30] of char;
  FHandle: Integer;
begin
  erCode := 0;
  Result := True;
  CheckWrite := FileMode and fmOpenWrite = fmOpenWrite;
  StrPCopy(FName, Drive+':\~~~a.$$$');
  OldErrorMode := SetErrorMode(SEM_FAILCRITICALERRORS);
  try
    if CheckWrite then FHandle := OpenFile(FName, OpenBuff, of_Create)
    else FHandle := OpenFile(FName, OpenBuff, of_Exist);
    erCode := OpenBuff.nErrCode;
    Result := (erCode <> 21);
    if CheckWrite then
      Result := Result and (erCode <> 19);
    if not Result then
      StrPCopy(Msg, Format('Drive %s: is %s', [Drive, Csts[OpenBuff.nErrCode <> 21]]));
    if FHandle <> HFILE_IERROR then
      _lClose(FHandle);
    if CheckWrite then
      OpenFile(FName, OpenBuff, of_Delete);
  finally
    SetErrorMode(OldErrorMode);
    if not Result then
      Abort := MessageBox(0, Msg, 'I/O Error', MB_ICONQUESTION or MB_RETRYCANCEL) = IDCANCEL
    else
      Abort := False;
  end;
end;

function jFindFirst(const Path: string; Attr: Word; var SearchRec: TSearchRec;
  FileMode: Integer): Integer;
var
  Ok, Abort: Boolean;
begin
  Abort := False;
  FillChar(SearchRec, SizeOf(SearchRec), #0);
  repeat
    Ok := DriveIsReady(Path[1], FileMode, Abort, Result);
    if Ok then
      Result := FindFirst(Path, Attr, SearchRec);
  until Ok or Abort;
  if Abort then
    Raise EUserAborted.Create('Canceled');
end;

function LongFormat2Str(Long: Longint): String;
begin
  Result := Format('%d', [Long]);
  if abs(Long) >= 1000 then Insert(',', Result, Length(Result) - 2);
  if abs(Long) >= 1000000 then Insert(',', Result, Length(Result) - 6);
  if abs(Long) >= 1000000000 then Insert(',', Result, Length(Result) - 10);
end;

function GetDrive_DiskFree(Drive: Byte): Longint;
begin
  Result := MaxLongint;
  if (Drive in [0..26]) then
    if DiskSize(Drive) > 0 then
      Result := DiskFree(Drive);
  if Result < 0 then Result := MaxLongint;
end;

function OperatingSystem: String;
var
  ssd, sv, sb: String;
begin
  if WinVer = W98 then
    {$IFDEF WIN32} Result := 'Microsoft Windows98 Operating System'
    {$ELSE}        Result := Format('Microsoft Windows98 (MS-DOS %d.%d)',
			     [HiByte(HiWord(GetVersion)), LoByte(HiWord(GetVersion))])
    {$ENDIF}
  else

  if WinVer = W95 then
    {$IFDEF WIN32} Result := 'Microsoft Windows95 Operating System'
    {$ELSE}        Result := Format('Microsoft Windows95 (MS-DOS %d.%d)',
			     [HiByte(HiWord(GetVersion)), LoByte(HiWord(GetVersion))])
    {$ENDIF}
  else

  if IsNT then
  begin
    {$IFDEF WIN32}
    Result := 'Microsoft Windows NT';
    with TRegistry.Create do
    try
      RootKey := HKEY_LOCAL_MACHINE;
      if OpenKey('SOFTWARE\Microsoft\Windows NT\CurrentVersion', False) then
      begin
        sv  := ReadString('CurrentVersion');
        sb  := ReadString('CurrentBuildNumber');
        ssd := ReadString('CSDVersion');
    {$ELSE}
    Result := 'Microsoft Windows NT Operating System';
    with TRegistry1632.Create do
    try
      sv  := ReadString(HKEY_LOCAL_MACHINE, NTCurrentVersionKey, 'CurrentVersion');
      sb  := ReadString(HKEY_LOCAL_MACHINE, NTCurrentVersionKey, 'CurrentBuildNumber');
      ssd := ReadString(HKEY_LOCAL_MACHINE, NTCurrentVersionKey, 'CSDVersion');
    {$ENDIF}
        if sv <> '' then begin
          Result := Format('%s'#10'    Version %s', [Result, sv]);
          if (sb <> '') or (ssd <> '') then begin
            Result := Format('%s (', [Result]);
            if (sb <> '') then begin
              Result := Format('%sBuild %s', [Result, sb]);
              if ssd <> '' then Result := Format('%s: ', [Result]);
            end;
            if ssd <> '' then Result := Format('%s%s', [Result, ssd]);
            Result := Format('%s)', [Result]);
          end
        end;
      {$IFDEF WIN32}
      end;
      {$ENDIF}
    finally
      Free;
    end
  end
  else
    Result := Format('Windows %d.%d (MS-DOS %d.%d)',
		[LoByte(WinVersion), HiByte(WinVersion),
	   	 HiByte(HiWord(GetVersion)), LoByte(HiWord(GetVersion))]);
end;

{$IFDEF WIN32}
var
  SystemInfo: TSystemInfo;
  OSVersionInfo: TOSVersionInfo;
{$ENDIF}
initialization
  IsWin32  := False;
  IsWin32s := False;
  IsNT     := False;
  IsWin95  := False;
  IsWin98  := False;
  IsWin31  := True;

  {$IFDEF WIN32}
  IsWin31 := False;
  IsWin32 := True;
  OSVersionInfo.dwOSVersionInfoSize := SizeOf(OSVersionInfo);
  GetVersionEx(OSVersionInfo);
  GetSystemInfo(SystemInfo);

  IsWin32s := OSVersionInfo.dwPlatformId = 0;
  IsWin95 := OSVersionInfo.dwPlatformId = 1;
  IsNT := OSVersionInfo.dwPlatformId = 2;

  if IsNT then WinVer := WNT
  else
    if IsWin95 then WinVer := W95
    else
      if IsWin32s then WinVer := W31
      else WinVer := W30; {!?!?!? seems imposible to me: there is no delphi 32 version for
			   Windows 3.0. or maybe something wrong in our evaluations?}
  IsWin98 := IsWin95 and (OSVersionInfo.dwMajorVersion = 4) and
                         (OSVersionInfo.dwMinorVersion = 10);
  IsWin95 := IsWin95 and not IsWin98;
  if IsWin98 then WinVer := W98;
  if IsNT then
    WinVersion := MakeWord(Byte(OSVersionInfo.dwMajorVersion), Byte(OSVersionInfo.dwMinorVersion))
  else ;
  {$ELSE}
  WinVersion := LoWord(GetVersion);
  if GetWinFlags and $4000 = $4000 then WinVer := WNT
  else
    {if GetWinFlags and $2000 = $2000 then WinVer := W??
    else dunno what a hell is $2000 for }
     if WinVersion = $0003 then WinVer := W30
     else
       if WinVersion = $0A03 then WinVer := W31
       else
         if WinVersion = $5F03 then WinVer := W95
         else
           IsWin32s := HiByte(HiWord(GetVersion)) = 1;

  IsWin95 := WinVer = W95;
  IsWin98 := IsWin95 and (HiByte(HiWord(GetVersion)) = 7) and
                         (LoByte(HiWord(GetVersion)) = 10);
  IsWin95 := IsWin95 and not IsWin98;
  if IsWin98 then WinVer := W98;
  IsNT := WinVer = WNT;
  {$ENDIF}
  IsWin31 := WinVer = W31;

  UserAbort := False;
  WantException := False;
  DOSError:=0;
end. { END UNIT }

The information provided above is provided "AS IS" without warranty of any kind.
Lucian Radulescu disclaims all warranties, either express or implied, including
the warranties of merchantability and fitness for a particular purpose.

In no event shall Lucian Radulescu or his suppliers be liable for any damages
whatsoever including direct, indirect, incidental, consequential, loss of business
profits or special damages, even if Lucian Radulescu or his suppliers have been
advised of the possibility of such damages.

Some states do not allow the exclusion or limitation of liability for consequential
or incidental damages so the foregoing limitation may not apply.

