{*******************************************************}
{                                                       }
{  JMemory: copying memory between overlapped src-dest  }
{  ---------------------------------------------------  }
{  ADV Compiled                                         }
{  Copyright (C), Lucian Radulescu                      }
{  E-mail:    ljee@interlog.com                         }
{  Home Page: http://www.interlog.com/~ljee             }
{                                                       }
{  Last reviewed: April 17th, 1999                      }
{                                                       }
{*******************************************************}

{$I ALLLIB.DEF}

unit JMemory;

interface

uses
  {$IFDEF WIN32} Windows,
  {$ELSE}        WinTypes, WinProcs,
  {$ENDIF}       SysUtils;

type
  TCopyMemProc = procedure(PDest, PSource: Pointer; Len: Longint);

{$IFNDEF VER120} {$IFNDEF VER100}
function CompareMem(P1, P2: Pointer; Length: Integer): Boolean;
{$ENDIF VER100} {$ENDIF VER120}

{$IFDEF WIN32}
procedure hmemcpy(P1, P2: Pointer; L: Longint);
{ hmemcpy replaced by MoveMemory in 32 bits environment. }
{$ENDIF}
function memcpy(Source: THandle): THandle;

function LocatePtr(Mem: Pointer; Index: LongInt): Pointer;
{ LocatePtr converts the offset "index" of "Mem" into a valid pointer.   }
{ Mem is retrieved with a previous GlobalLock.                           }
{ Index is the desired offset from THE BEGINING of block (byte zero).    }

function OffsPtr(Mem: Pointer; Index: LongInt): Pointer;
{ OffsPtr converts the offset "index" of "Mem" into a valid pointer.     }
{ Mem is retrieved with a previous GlobalLock and possibly relocated,    }
{ with LocatePtr or OffsPtr (or anything else for "walk" through memory).}
{ Index is the desired offset from CURRENT POSITION.                     }

procedure AssignMemCopyProc(FastCopy: Boolean);
var SafeMemCopy: TCopyMemProc;

implementation

{$IFDEF WIN32}
procedure hmemcpy(P1, P2: Pointer; L: Longint);
begin
  MoveMemory(P1, P2, L);
end;
{$ENDIF}

function memcpy(Source: THandle): THandle;
var
  Src, Dest: Pointer;
  Size: Integer;
begin
  if Source <> 0 then
  begin
    Size := GlobalSize(Source);
    Result := GlobalAlloc(GHND, Size);
    if Result <> 0 then
      try
        Src := GlobalLock(Source);
        Dest := GlobalLock(Result);
        if (Src <> nil) and (Dest <> nil) then hmemcpy(Dest, Src, Size);
      finally
        GlobalUnlock(Source);
        GlobalUnlock(Result);
      end
  end
  else Result := 0;
end;

function OffsPtr(Mem: Pointer; Index: LongInt): Pointer;
begin
  Result := Pointer(LongInt(Mem) + Index);
end;

function LocatePtr(Mem: Pointer; Index: LongInt): Pointer;
{$IFDEF WIN32}
var
  HMem: THandle;
begin
  HMem := GlobalHandle(Mem);
  GlobalUnlock(HMem);
  Result := OffsPtr(GlobalLock(HMem), Index);
end;
(* also works next code:
assembler;
asm
	PUSH    EDX
	PUSH    EAX
	CALL    GlobalHandle
	MOV     ECX,EAX
	PUSH    EAX
	CALL    GlobalUnlock
	PUSH    ECX
	CALL    GlobalLock
	POP     EDX
	ADD	EAX,EDX
end;
*)
{$ELSE}
var
  HMem: THandle;
  Selector, OffSet: Word;
begin
  HMem := GlobalHandle(PtrRec(Mem).Seg);
  Selector := (Index div 65536) * 8 + HMem;
  Offset := Index mod 65536;
  Result := Ptr(Selector, Offset);
end;
{$ENDIF}

procedure procSafeMemCopy(PDest, PSource: Pointer; Len: Longint);{$IFNDEF WIN32}FAR;{$ENDIF}
{ SafeMemCopy provides a safe hMemCpy equiv.
    PDest: Points to a buffer that receives the copied bytes.
    PSource: Points to a buffer that contains the bytes to be copied.
    Len: Specifies the number of bytes to be copied.

  There are four cases to consider:
    case 1: source and destination are the same;
    case 2: source and destination do not overlap;
    case 3: source starts at a location before destination in linear memory;
            (make sure there is enough allocated room to relocate Len bytes.)
    case 4: source starts at a location after destination in linear memory.

***
  This code translated for Delphi from Microsoft knowledge base:
      "Last reviewed: July 23, 1997
       Article ID: Q117743 3.10 WINDOWS kbprg"

    in MS's source code there is a bug in case 3 detection:
       if ((s < d) && ((unsigned long)s+len-1) >(unsigned long)d) // original

    this is how it should be:
       if ((s < d) && ((unsigned long)s+len) >(unsigned long)d)   // fixed: wrong "-1" gone

  Note: in Win32 this function just calls MoveMemory, now safe with overlapping
	memory (Microsoft's fixed their bugg...)

  If your concern it's speed despite memory, define FastMemCopy in your project options.
}
var
  Times, I, SafeSize: Longint;
begin
  if (PDest = PSource) then Exit;  { case 1 }

  SafeSize := ABS(Longint(PDest)-Longint(PSource));{ determine the amount of overlap }
  if (SafeSize >= Len) then        { case 2 }
  begin
    hMemCpy(PDest, PSource, Len);  { no overlap }
    Exit;
  end;
  Times := Len div SafeSize;
  if (Longint(PDest) > Longint(PSource)) and (Longint(PSource) + Len > Longint(PDest)) then
  begin { case 3 }
    { copy bytes from the end of source to the end of destination in safesize quantum. }
    for I := 1 to Times do
      hMemCpy(OffsPtr(PDest, Len-I*SafeSize), OffsPtr(PSource, Len-I*SafeSize), SafeSize);
    { copy the bytes remaining to be copied after times*safesize bytes have been copied. }
    if (Times*SafeSize < Len) then
      hMemCpy(PDest, PSource, Len-Times*SafeSize);
  end
  else
  begin { case 4 }
    { copy bytes from the beginning of source to the beginning of destination in safesize quantum }
    for I:= 0 to Times-1 do
      hMemCpy(OffsPtr(PDest, I*SafeSize), OffsPtr(PSource, I*SafeSize), SafeSize);
    { copy the bytes remaining to be copied after times*safesize bytes have been copied. }
    if (Times*Safesize < Len) then
      hMemCpy(OffsPtr(PDest, Times*SafeSize), OffsPtr(PSource, Times*SafeSize), Len-Times*SafeSize);
  end;
end;

procedure procFastMemCopy(PDest, PSource: Pointer; Len: Longint); {$IFNDEF WIN32}FAR;{$ENDIF}
var
  FMemTemp: Pointer;
begin
  FMemTemp := GlobalAllocPtr(GHND, Len); { allocate temporary memory    }
  hMemCpy(FMemTemp, PSource, Len);       { copy to temporary memory     }
  hMemCpy(PDest, FMemTemp, Len);         { copy to destination addresss }
  GlobalFreePtr(FMemTemp);               { free temporary memory        }
end;            {You might want to do the above in a try-finally block. }

procedure AssignMemCopyProc(FastCopy: Boolean);
{provided in the interface part, so you can change it at run time}
begin
  if FastCopy then SafeMemCopy := procFastMemCopy
  else SafeMemCopy := procSafeMemCopy;
end;

{$IFNDEF VER120} {$IFNDEF VER100}
  {$IFDEF WIN32}
  function CompareMem(P1, P2: Pointer; Length: Integer): Boolean; assembler;
  asm
          PUSH    ESI
          PUSH    EDI
          MOV     ESI,P1
          MOV     EDI,P2
          MOV     EDX,ECX
          XOR     EAX,EAX
          AND     EDX,3
          SHR     ECX,2
          REPE    CMPSD
          JNE     @@2
          MOV     ECX,EDX
          REPE    CMPSB
          JNE     @@2
  @@1:    INC     EAX
  @@2:    POP     EDI
          POP     ESI
  end;
  {$ELSE}
  function CompareMem(P1, P2: Pointer; Length: Integer): Boolean; assembler;
  asm
          PUSH DS         { Save DS }
          MOV  AX, 1      { Result: True }
          MOV  CX, Length { Bufer size in CX }
          JCXZ @@DONE     { Finnished if size is zero }
          LDS  SI, P1     { DS:SI points to P1 }
          LES  DI, P2     { ES:DI points to P2 }
          CLD             { Flag to bump SI, DI }
          REP  CMPSB      { compare buffers }
          JE   @@Done     { done if equals }
          XOR  AX, AX     { Result: False }
  @@DONE: POP  DS	  { Restore DS }
  end;
  {$ENDIF WIN32}
{$ENDIF VER100} {$ENDIF VER120}

initialization
  {$IFDEF FASTMEMCOPY} AssignMemCopyProc(True);
  {$ELSE}              AssignMemCopyProc(False);
  {$ENDIF}
end.

The information provided above is provided "AS IS" without warranty of any kind.
Lucian Radulescu disclaims all warranties, either express or implied, including
the warranties of merchantability and fitness for a particular purpose.

In no event shall Lucian Radulescu or his suppliers be liable for any damages
whatsoever including direct, indirect, incidental, consequential, loss of business
profits or special damages, even if Lucian Radulescu or his suppliers have been
advised of the possibility of such damages.

Some states do not allow the exclusion or limitation of liability for consequential
or incidental damages so the foregoing limitation may not apply.

