{*******************************************************}
{                                                       }
{       MRU Files Interface                             }
{       -------------------                             }
{       ADV Compiled                                    }
{                                                       }
{*******************************************************}

unit MRUFiles;

{
  CONDITIONAL DEFINES CHECKED FOR TRUE:
     WIN32
  CONDITIONAL DEFINES CHECKED FOR FALSE:
}

{$I ALLLIB.DEF}

interface

uses
  {$IFDEF WIN32} Windows,
  {$ELSE}        WinTypes, WinProcs,
  {$ENDIF}
  SysUtils, Classes, Menus, IniFiles, FileCtrl, FileTool, LongColl;

const

  ctMaxRecentFiles = 10;

type

  EMRU          = class(Exception);
  EMRU_NotFound = class(EMRU);

  TCheckFileTypeFunc = function (const FileName: String): String;

  { TMRUFiles }
  TMRUFiles = class(TCollection32)
  private
    { Private declarations }
    FIniFileName: String;
    FMaxRecentFiles: Integer;
    function GetFileName(Index: Longint): String;
    function StringAsPChar(var S: String): PChar;
  protected
    { Protected declarations }
    procedure FreeItem(Item: Pointer); override;
  public
    { Public declarations }
    constructor Create(IniFileName: String);
    procedure Clear;
    procedure CheckIt;
    function Equal(AIndex: Integer; AFileName: String): Boolean;
    procedure SetIni(IniFile: TIniFile);
    procedure SetIniFile(IniFileName: String);
    procedure AssignStrings(S: TStrings);
    procedure AddRecentFile(AFileName: String; UpdateCount: Boolean);
    procedure DeleteRecentFile(AFileName: String);
    procedure Erase_File(AFileName: String);
    procedure LoadMRU; virtual;
    procedure StoreMRU; virtual;
    procedure UpdateFileMenu(FileMenu, BreakMenu: TMenuItem; Func: TCheckFileTypeFunc);
    property FileName[Index: Longint]: String read GetFileName;
    property MaxRecentFiles: Integer read FMaxRecentFiles write FMaxRecentFiles;
  end;

implementation

{ TMRUFiles }
constructor TMRUFiles.Create(IniFileName: String);
begin
  inherited Create(ctMaxRecentFiles, 0);
  FIniFileName := IniFileName;
  FMaxRecentFiles := ctMaxRecentFiles;
end;

procedure TMRUFiles.Clear;
begin
  FreeAllItems;
end;

function TMRUFiles.Equal(AIndex: Integer; AFileName: String): Boolean;
begin
  Result := CompareText(AFileName, FileName[AIndex]) = 0;
end;

procedure TMRUFiles.SetIni(IniFile: TIniFile);
begin
  FIniFileName := IniFile.FileName;
end;

procedure TMRUFiles.SetIniFile(IniFileName: String);
begin
  FIniFileName := IniFileName;
end;

procedure TMRUFiles.FreeItem(Item: Pointer);
begin
  if Assigned(Item) then
    StrDispose(PChar(Item));
end;

function TMRUFiles.GetFileName(Index: Longint): String;
begin
  Result := StrPas(Items[Index]);
end;

function TMRUFiles.StringAsPChar(var S: String): PChar;
begin
  {$IFDEF WIN32}
  Result := PChar(S);
  {$ELSE}
  if Length(S) = High(S) then Dec(S[0]); { Truncate S if it's too long }
  S[Ord(Length(S))+1] := #0;             { Place null at end of string }
  Result := @S[1];                       { Return "PChar'd" string }
  {$ENDIF}
end;

procedure TMRUFiles.AssignStrings(S: TStrings);
var
  I: Integer;
begin
  Clear;
  for I := 0 to S.Count - 1 do
    AddRecentFile(S[I], False);
end;

procedure TMRUFiles.DeleteRecentFile(AFileName: String);
var
  TheName: String;

function SearchIt(P: PChar): Boolean; far;
begin
  Result := StrIComp(P, StringAsPChar(TheName)) = 0;
end;

begin
  TheName := LowerCase(AFileName);
  if FirstThat(@SearchIt) <> nil then
    FreeAtIndex(Curr);
end;

procedure TMRUFiles.Erase_File(AFileName: String);
begin
  DeleteFile(AFileName);
  DeleteRecentFile(AFileName);
end;

procedure TMRUFiles.AddRecentFile(AFileName: String; UpdateCount: Boolean);
var
  I: Integer;
  Res: Word;
begin
  AFileName := LowerCase(AFileName);
  {Res := GetDriveType(Ord(UpCase(AFileName[1]))-65);
  if (Res = 0) or (Res = DRIVE_REMOVABLE) then Exit;}
  if Ord(UpCase(AFileName[1]))-65 < 2 then Exit; {a: b: floppy don't update recent files}
  DeleteRecentFile(AFileName);
  if GetFileSize(AFileName) = 0 then DeleteFile(AFileName);
  if not FileExists(AFileName) then Exit;
  if Count = FMaxRecentFiles then FreeAtIndex(Count-1);
  if UpdateCount then I := 0
  else I := Count;
  AtInsert(I, StrNew(StringAsPChar(AFileName)));
end;

procedure TMRUFiles.CheckIt;
var
  I: Integer;
begin
  if Count = 0 then Exit;
  I := 0;
  while I < Count do
    if not DirectoryExists(ExtractFilePath(StrPas(Items[I]))) or
       not FileExists(StrPas(Items[I])) or
       (Pos('~~', ExtractFileName(StrPas(Items[I]))) <> 0) then
       DeleteRecentFile(StrPas(Items[I]))
    else Inc(I);
end;

procedure TMRUFiles.LoadMRU;
var
  I: Integer;
  S: TStringList;
begin
  Clear;
  S := TStringList.Create;
  try
    with TIniFile.Create(FIniFileName) do
    try
      ReadSectionValues('MRU Files', S);
    finally
      Free;
    end;
    for I := 0 to S.Count - 1 do
      AddRecentFile(S.Values[Format('File%d', [I])], False);
  finally
    S.Free;
  end;
end;

procedure TMRUFiles.StoreMRU;
var
  Ini: TIniFile;

procedure WriteIniLine(AFileName: PChar); far;
begin
  Ini.WriteString('MRU Files', Format('File%d', [Curr]), StrPas(AFileName));
end;

begin
  Ini := TIniFile.Create(FIniFileName);
  with Ini do
  try
    EraseSection('MRU Files');
    ForEach(@WriteIniLine);
  finally
    Free;
  end;
end;

procedure TMRUFiles.UpdateFileMenu(FileMenu, BreakMenu: TMenuItem;
  Func: TCheckFileTypeFunc);
var
  MenuItem: TMenuItem;
  BP, I: Integer;
begin
  CheckIt;
  BP := FileMenu.IndexOf(BreakMenu);
  if BP <> -1 then
  begin
    for I := 1 to 10 do
      FileMenu.Items[BP + I].Visible := False;
    FileMenu.Items[BP].Visible := Count > 0;
    if Count > 0 then
    begin
      I := 0;
      repeat
	MenuItem := FileMenu.Items[BP + I + 1];
	if MenuItem <> nil then
	begin
	  MenuItem.Caption := Format('&%d %s', [I + 1, Func(FileName[I])]);
	  MenuItem.Visible := True;
	end;
	Inc(I);
      until (I = 10) or (I = Count);
    end;
  end;
end;

end.

