{***********************************************************}
{                                                           }
{     Streams                                               }
{     =======                                               }
{     ADV Compiled                                          }
{     Copyright  1995-1999 by Lucian Radulescu             }
{     E-mail    : ljee@interlog.com                         }
{     Home Page : http://www.interlog.com/~ljee             }
{                                                           }
{     Developped on March 1998, provides same features as   }
{     old BP7 TBufStream, reading/writing using a buffer.   }
{     File status: FREEWARE                                 }
{                  (you may use it, but not sell it)        }
{     Last time reviewed: Oct. 27th, 1999.                  }
{                                                           }
{***********************************************************}

{$I ALLLIB.DEF}

unit Streams;

interface

uses
  {$IFDEF WIN32} Windows,
  {$ELSE}        WinTypes, WinProcs,
  {$ENDIF}
  SysUtils, Classes {$IFDEF STREAMS32}, FStream {$ENDIF STREAMS32}
  ;

const
  MaxBufSize = $8000; {=32KB}

type

  {$IFNDEF WIN32}
  DWORD = Longint;
  {$ENDIF WIN32}

  TOnCopyFile  = procedure(Sender: TObject; FileName: String; Size: Longint) of object;
  TOnProgress  = procedure(Sender: TObject; Position: Longint) of object;
  TCallBackSet = procedure(Sender: TObject) of object;

  TFlushMode   = (tfRead, tfWrite, tfBoth);
  TStreamOperation = (topRead, topWrite);

  { TBufStream }
  TBufStream   = class(TFileStream)

  private

    { Private declarations }

    Buffer       : Pointer;   { our buffer }
    BufEnd       : Cardinal;  { end of buffer; used on reading }
    BufPtr       : Cardinal;  { actual position inside buffer; used by both read/write }
    BufSize      : Cardinal;  { buffer size }
    Operation    : TStreamOperation;

    FileName     : String;    { current file name }

    FOnBeginCopy : TOnCopyFile;
    FOnEndCopy   : TNotifyEvent;
    FOnProgress  : TOnProgress;

    {function  AbortProc: Boolean;}
    function  GetPosition: Longint;
    procedure SetPosition(Value: Longint);
    function  GetSize: Longint;

  protected

    { Protected declarations }

    WorkBytes : Cardinal;

    procedure  WriteBuffer;

    procedure BeginCopy(FileName: String; Size: Longint); dynamic;
    procedure EndCopy; dynamic;
    procedure Progress(Position: Longint); dynamic;

  public

    { Public declarations }

    Err_No_More_Data: Boolean; {June 19th}
    WantExcept: Boolean;       {June 19th}

    constructor Create(const AFileName: String; Mode: Word; Size: Cardinal
      {$IFDEF STREAMS32}; dwFlags: DWORD{$ENDIF STREAMS32});
    constructor CreateFrom(InputStream: TBufStream; const AFileName: String
      {$IFDEF STREAMS32}; dwFlags: DWORD{$ENDIF STREAMS32});
    destructor Destroy; override;

    {made next avialable for outside use}
    procedure  FlushBuffer(FlushMode: TFlushMode);
    function   ReadBuffer(Count: Cardinal): Longint;

    function CopyFrom(Source: TBufStream; Count: Longint): Longint;
    function CopyFromStream(Source: TStream; Count: Longint): Longint;
    function Read(var Buf; Count: Longint): Longint; override;
    function Write(const Buf; Count: Longint): Longint; override;
    function Seek(Offset: Longint; Origin: Word): Longint; override;

    procedure Skip(Count: Cardinal); {May 5, 1999}

    {Apr. 22, 1999}
    procedure StrWriteEx(P: PChar);
    procedure WriteStrEx(Str: String);
    procedure WriteLine(Line: String);

    { Properties }

    property Buf: Pointer read Buffer;
    property LastRead: Cardinal read BufEnd;
    property Position: Longint read GetPosition write SetPosition;
    property Size: Longint read GetSize;

    property OnBeginCopyFile: TOnCopyFile read FOnBeginCopy write FOnBeginCopy;
    property OnEndCopyFile: TNotifyEvent read FOnEndCopy write FOnEndCopy;
    property OnProgress: TOnProgress read FOnProgress write FOnProgress;

  end;

procedure StreamCopy(const InFileName, OutFileName: String; SetEvents: TCallBackSet);

implementation

uses
  EQ_Abort;

type
  TBytes = array [0..MaxBufSize-1] of Byte;

procedure StreamCopy(const InFileName, OutFileName: String; SetEvents: TCallBackSet);
var
  InFile, OutFile: TBufStream;
begin
  UserAbort := False;
  if not FileExists(InFileName) then Exit;
  InFile := TBufStream.Create(InFileName, fmOpenRead, MaxBufSize
     {$IFDEF STREAMS32}, FILE_ATTRIBUTE_NORMAL{$ENDIF STREAMS32});
  try
    if FileExists(OutFileName) then
      DeleteFile(OutFileName);
    OutFile := TBufStream.CreateFrom(InFile, OutFileName
      {$IFDEF STREAMS32}, FILE_ATTRIBUTE_NORMAL{$ENDIF STREAMS32});
    try
      if Assigned(SetEvents) then SetEvents(OutFile);
      OutFile.CopyFrom(InFile, 0);
    finally
      OutFile.Free;
    end;
  finally
    InFile.Free;
  end;
end;

{ TBufStream }
constructor TBufStream.Create;
begin
  inherited Create(AFileName, Mode{$IFDEF STREAMS32}, dwFlags{$ENDIF STREAMS32});
  FileName  := AFileName;
  BufEnd    := 0;
  BufPtr    := 0;
  if Size > MaxBufSize-1 then Size := MaxBufSize-1;
  BufSize := Size;
  Buffer  := nil;
  WantExcept := True; {June 19th}
  if BufSize > 0 then GetMem(Buffer, BufSize)
  else
    raise EFCreateError.Create('STREAMS: Zero size not allowed here!');
end;

constructor TBufStream.CreateFrom(InputStream: TBufStream; const AFileName: String
  {$IFDEF STREAMS32}; dwFlags: DWORD{$ENDIF STREAMS32});
begin
  inherited Create(AFileName, fmCreate{$IFDEF STREAMS32}, dwFlags{$ENDIF STREAMS32});
  FileName  := AFileName;
  BufEnd    := 0;
  BufPtr    := 0;
  Buffer    := nil;
  WantExcept := True; {June 19th}
  if not Assigned(InputStream) then
    raise EFCreateError.Create('STREAMS: InputStream not allowed to be NIL!');
  BufSize   := InputStream.BufSize;
end;

destructor TBufStream.Destroy;
begin
  FlushBuffer(tfBoth);
  if Assigned(Buffer) then FreeMem(Buffer, BufSize);
  inherited Destroy;
end;

procedure TBufStream.FlushBuffer(FlushMode: TFlushMode);
begin
  if FlushMode = tfRead then
  begin
    if BufEnd <> 0 then BufPtr := 0; {only if I was reading}
    BufEnd := 0;
  end else
  begin
    if (BufEnd = 0) and (BufPtr > 0) then WriteBuffer; {only if I was writing}
    if (BufEnd <> 0) and (FlushMode = tfBoth) then     {only if I was reading}
    begin                                              {and tfBoth is requested}
      BufPtr := 0;
      BufEnd := 0;
    end;
  end;
end;

procedure TBufStream.BeginCopy(FileName: String; Size: Longint);
begin
  if Assigned(FOnBeginCopy) then FOnBeginCopy(Self, FileName, Size);
  AbortProc;
end;

procedure TBufStream.EndCopy;
begin
  if Assigned(FOnEndCopy) then FOnEndCopy(Self);
  AbortProc;
end;

procedure TBufStream.Progress(Position: Longint);
begin
  if Assigned(FOnProgress) then FOnProgress(Self, Position);
  AbortProc;
end;

{June 22, 1999: tested & fixed}
function TBufStream.CopyFromStream(Source: TStream; Count: Longint): Longint;

  function Inc(var X: Cardinal; N: Longint): Longint;
  begin
    Result := N;
    X := X + Result;
  end;

var
  N: Cardinal;
begin
  if Count = 0 then
  begin
    Count := Source.Size;
    Source.Position := 0;
  end;
  Result  := Count;
  BeginCopy('', Result);
  try
    while not UserAbort and (Count <> 0) do
    begin
      if Count > BufSize-BufPtr then N := BufSize-BufPtr
      else N := Count;
      N := Source.Read(TBytes(Buffer^)[BufPtr], N);
      Dec(Count, Inc(BufPtr, N));
      WriteBuffer;
      Progress(Result-Count);
    end;
  finally
    EndCopy;
  end;
end;

{June 22, 1999: tested & fixed}
function TBufStream.CopyFrom(Source: TBufStream; Count: Longint): Longint;

  function Inc(var X: Cardinal; N: Longint): Longint;
  begin
    Result := N;
    X := X + Result;
  end;

var
  N, StartPtr: Cardinal;
begin
  if Count = 0 then
  begin
    Count := Source.Size;
    Source.Position := 0;
  end;
  Result  := Count;
  BeginCopy(FileName, Result);
  try
    while not UserAbort and (Count <> 0) do
    begin
      StartPtr := Source.BufPtr;
      if Count > Source.BufSize - StartPtr then
        N := Source.BufSize - StartPtr
      else N := Count;

      if N > BufSize - BufPtr then
        N := BufSize - BufPtr;

      N := Source.ReadBuffer(N);
      Move(TBytes(Source.Buffer^)[StartPtr], TBytes(Buffer^)[BufPtr], N);
      Dec(Count, Inc(BufPtr, N));
      Inc(Source.BufPtr, N);
      { Oct. 12, 1999 -------------------------- }
      if Source.BufEnd - Source.BufPtr = 0 then
      begin
        Source.BufEnd := 0;
        Source.BufPtr := 0;
      end;
      { Oct. 12, 1999 -------------------------- }
      WriteBuffer;
    end;
    if not UserAbort then FileSetDate(Handle, FileGetDate(Source.Handle));
  finally
    EndCopy;
  end;
end;

{ We read/write in a "repeat until" loop because }
{ count might be bigger than our buffer size.    }

function TBufStream.Read(var Buf; Count: Longint): Longint;
begin
  FlushBuffer(tfWrite);
  Operation := topRead;
  Result := 0;
  if Count = 0 then Exit;
  repeat
    WorkBytes := ReadBuffer(Count);
    Move(TBytes(Buffer^)[BufPtr], TBytes(Buf)[Result], WorkBytes);
    Inc(BufPtr, WorkBytes);
    Inc(Result, WorkBytes);
    Dec(Count,  WorkBytes);
  until Err_No_More_Data or (Count = 0); {June 19th}
end;

function TBufStream.ReadBuffer(Count: Cardinal): Longint;
var
  BytesLeft: Cardinal;
  BytesDone: DWORD;
begin
  Err_No_More_Data := False; {June 19th}
  Result := Count;
  if Count = 0 then Exit;
  BytesLeft := BufEnd - BufPtr;
  if (BufPtr < BufEnd) and (Count > BytesLeft) then
    Move(TBytes(Buffer^)[BufPtr], TBytes(Buffer^)[0], BytesLeft)
  else BytesLeft := 0;
  if (BufEnd = 0) or (BufPtr = BufEnd) or (BytesLeft > 0) then
  begin
    BytesDone := inherited Read(TBytes(Buffer^)[BytesLeft], BufSize-BytesLeft);
    Err_No_More_Data := BytesDone = 0; {June 19th}
    if Err_No_More_Data then           {June 19th}
      if WantExcept then               {June 19th}
        raise EStreamError.Create('STREAMS: Stream read error.');
    BufEnd := BytesDone + BytesLeft;
    BufPtr := 0;
    if BufEnd < Count then Result := BufEnd;
  end;
end;

function TBufStream.Write(const Buf; Count: Longint): Longint;
begin
  FlushBuffer(tfRead);
  Operation := topWrite;
  Result := 0;
  if Count = 0 then Exit;
  repeat
    WorkBytes := BufSize - BufPtr;
    if WorkBytes = 0 then WriteBuffer
    else
    begin
      if (Count < WorkBytes) then WorkBytes := Count;
      Move(TBytes(Buf)[Result], TBytes(Buffer^)[BufPtr], WorkBytes);
      Inc(BufPtr, WorkBytes);
      Inc(Result, WorkBytes);
      Dec(Count,  WorkBytes);
    end;
  until Count = 0;
end;

procedure TBufStream.WriteBuffer;
begin
  inherited Write(Buffer^, BufPtr);
  BufPtr := 0;
end;

procedure TBufStream.Skip(Count: Cardinal);
{var
  P: Pointer;}
begin
{
  case Operation of
    topRead: begin
               GetMem(P, Count);
               try     Read(P^, Count);
               finally FreeMem(P, Count);
               end;
             end;
    topWrite: ;
  end;
}
  {***   WARNING! WARNING! WARNING! NOT TESTED   ***}
  Seek(Count-BufEnd+BufPtr, soFromCurrent); {June 22, 1999}
end;

procedure TBufStream.StrWriteEx(P: PChar);
begin
  if Assigned(P) and (P^ <> #0) then Write(P^, StrLen(P));
end;

procedure TBufStream.WriteStrEx(Str: String);
begin
  Write({$IFDEF WIN32}PChar(Str)[0],
        {$ELSE}       Str[1],
        {$ENDIF}      Length(Str));
end;

procedure TBufStream.WriteLine(Line: String);
begin
  WriteStrEx(Format('%s'#13#10, [Line]));
end;

function TBufStream.GetSize: Longint;
var
  Pos: Longint;
begin
  Pos := FileSeek(Handle, 0, 1);
  Result := FileSeek(Handle, 0, 2);
  if Pos <> Result then FileSeek(Handle, Pos, 0);
  if Operation = topWrite then {June 22, 1999}
    Inc(Result, BufPtr);       {June 22, 1999}
end;

function TBufStream.GetPosition: Longint;
begin
   Result := FileSeek(Handle, 0, 1) - BufEnd + BufPtr;
end;

procedure TBufStream.SetPosition(Value: Longint);
begin
  Seek(Value, soFromBeginning);
end;

function TBufStream.Seek(Offset: Longint; Origin: Word): Longint;
begin
  if (Origin = soFromCurrent) and (Offset = 0) then Result := Position
  else
  begin
    FlushBuffer(tfBoth);
    Result := FileSeek(Handle, Offset, Origin);
  end;
end;

{ Note: if stream is open ONLY for read OR for write, TBufStream is very optimized.
	If stream is open for read AND write operations and your code
	requests many back & forth seek operations, than there might be some loss
	of speed, depending upon your code, cuz of flush operations.
}

end.

{     SAMPLE OF USE:
      ==============

var
  S: TBufStream;
begin
  S := TBufStream.Create(FileName, fmCreate, 8192);
  try
    S.Write(Longint_Variable, SizeOf(Longint));
  finally
    S.Free;
  end;

  S := TBufStream.Create(FileName, fmOpenRead, 8192);
  try
    S.Read(Longint_Variable, SizeOf(Longint));
  finally
    S.Free;
  end;
end;
}

(*

{ Classes }

       { SysUtils }       { WinTypes/Windows }
       fmOpenRead         OF_READ             = $0000;
       fmOpenWrite        OF_WRITE            = $0001;
       fmOpenReadWrite    OF_READWRITE        = $0002;
       fmShareCompat      OF_SHARE_COMPAT     = $0000;
       fmShareExclusive   OF_SHARE_EXCLUSIVE  = $0010;
       fmShareDenyWrite   OF_SHARE_DENY_WRITE = $0020;
       fmShareDenyRead    OF_SHARE_DENY_READ  = $0030;
       fmShareDenyNone    OF_SHARE_DENY_NONE  = $0040;
		  	  OF_PARSE            = $0100;
			  OF_DELETE           = $0200;
			  OF_VERIFY           = $0400;
			  OF_CANCEL           = $0800;
fmCreate = $FFFF;         OF_CREATE           = $1000;
			  OF_PROMPT           = $2000;
			  OF_EXIST            = $4000;
			  OF_REOPEN           = $8000;
*)

The information provided above is provided "AS IS" without warranty of any kind.
Lucian Radulescu disclaims all warranties, either express or implied, including
the warranties of merchantability and fitness for a particular purpose.

In no event shall Lucian Radulescu or his suppliers be liable for any damages
whatsoever including direct, indirect, incidental, consequential, loss of business
profits or special damages, even if Lucian Radulescu or his suppliers have been
advised of the possibility of such damages.

Some states do not allow the exclusion or limitation of liability for consequential
or incidental damages so the foregoing limitation may not apply.


